/*********************************************************************************
  *Copyright(C),2015-2020, 
  *TUYA 
  *www.tuya.comm

  *FileName: tuya_ipc_dp_handler.c
  *
  * File Description：
  * 1. DP Point Setting and Acquisition Function API
  *
  * Developer work：
  * 1. Local configuration acquisition and update.
  * 2. Set local IPC attributes, such as picture flip, time watermarking, etc.
  *    If the function is not supported, leave the function blank.
  *
**********************************************************************************/
#include "utilities/uni_log.h"
#include "tuya_iot_config.h"
#include "tuya_ipc_stream_storage.h"
#include "tuya_ipc_ptz.h"
#include "tuya_ipc_move_path.h"
#include "ty_dp_define.h"
#include "ty_sdk_common.h"

#include "tkl_wifi.h"
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>

extern CHAR_T s_ipc_storage[64];

STATIC VOID __tuya_json_write(CHAR_T *buffer)
{
    CHAR_T config_path[256] = {0};
    snprintf(config_path, 256, "%s/tuya_dp_cfg.json", s_ipc_storage);

    FILE *fp = fopen(config_path, "w");
    fwrite(buffer, strlen(buffer), 1, fp); 
    fflush(fp);
    fclose(fp);
}

VOID TUYA_IPC_INIT_DP_CONFIG()
{
    CHAR_T config_path[256] = {0};
    INT_T fd = -1;
    INT_T sleep_mode = 0;
    INT_T light_switch = 0;
    INT_T flip_switch = 0;
    INT_T watermark_switch = 0;
    INT_T wdr_switch = 0;
    INT_T alarm_switch = 0;
    INT_T alarm_zone_switch = 0;
    INT_T sd_record_mode = 0;
    INT_T sd_record_switch = 0;
    INT_T blub_switch = 0;
    INT_T ap_switch = 0;
    CHAR_T night_mode[8] = {0};
    CHAR_T alarm_sen[8] = {0};
    CHAR_T ap_ssid[32] = {0};
    CHAR_T ap_passwd[32] = {0};

    snprintf(config_path, 256, "%s/tuya_dp_cfg.json", s_ipc_storage);
    if(0 == access(config_path, F_OK) ){
        return;
    }else{
        ty_cJSON *cfg_json = ty_cJSON_CreateObject();
        ty_cJSON_AddItemToObject(cfg_json, "tuya_sleep_mode", ty_cJSON_CreateNumber(sleep_mode));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_light_onoff", ty_cJSON_CreateNumber(light_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_flip_onoff", ty_cJSON_CreateNumber(flip_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_watermark_onoff", ty_cJSON_CreateNumber(watermark_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_wdr_onoff", ty_cJSON_CreateNumber(wdr_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_alarm_on_off", ty_cJSON_CreateNumber(alarm_switch));
        ty_cJSON_AddItemToObject(cfg_json, "alarm_zone_on_off", ty_cJSON_CreateNumber(alarm_zone_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_sd_record_mode", ty_cJSON_CreateNumber(sd_record_mode));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_sd_record_on_off", ty_cJSON_CreateNumber(sd_record_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_blub_on_off", ty_cJSON_CreateNumber(blub_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_ap_on_off", ty_cJSON_CreateNumber(ap_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_night_mode", ty_cJSON_CreateString(night_mode));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_alarm_sen", ty_cJSON_CreateString(alarm_sen));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_ap_ssid", ty_cJSON_CreateString(ap_ssid));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_ap_passwd", ty_cJSON_CreateString(ap_passwd));

        CHAR_T *buffer = ty_cJSON_Print(cfg_json);
        __tuya_json_write(buffer);
        ty_cJSON_free(buffer);
        ty_cJSON_Delete(cfg_json);
    }
}

STATIC ty_cJSON *__tuya_json_parse()
{
    CHAR_T buffer[4096] = {0};
    INT_T read_len = sizeof(buffer);
    INT_T fd = -1;
    CHAR_T config_path[256] = {0};

    snprintf(config_path, 256, "%s/tuya_dp_cfg.json", s_ipc_storage);
    fd = open(config_path, O_RDWR);
    if (fd < 0 ){
        PR_ERR("create %s failed \n", config_path);
        return NULL;
    }

    while (read_len == read(fd, buffer, sizeof(buffer)));
    close(fd);

    ty_cJSON *cfg_json = ty_cJSON_Parse(buffer);
    return cfg_json;
}


/* Setting and Getting Integer Local Configuration.
The reference code here is written directly to tmp, requiring developers to replace the key.*/
STATIC VOID __tuya_app_write_INT(CHAR_T *key, INT_T value)
{
    ty_cJSON *cfg_json = __tuya_json_parse();
    if (NULL == cfg_json){
        PR_ERR("load configure failed\n");
        return;
    }
    ty_cJSON_ReplaceItemInObject(cfg_json, key, ty_cJSON_CreateNumber(value));
    CHAR_T *json_tmp = ty_cJSON_Print(cfg_json);
    __tuya_json_write(json_tmp);
    ty_cJSON_free(json_tmp);
    ty_cJSON_Delete(cfg_json);
}

STATIC INT_T __tuya_app_read_INT(CHAR_T *key)
{
    int tmp_val = 0;
    ty_cJSON *cfg_json = __tuya_json_parse();
    if (NULL == cfg_json){
        PR_ERR("load configure failed\n");
        return OPRT_CJSON_PARSE_ERR;
    }

    ty_cJSON* val = ty_cJSON_GetObjectItem(cfg_json, key);
    if (val) {
        tmp_val = val->valueint;
    }
    ty_cJSON_Delete(cfg_json);
    return tmp_val;
}

/* Setting and Getting String Local Configuration.
The reference code here is written directly to tmp, requiring developers to replace the key.*/
STATIC VOID __tuya_app_write_STR(CHAR_T *key, CHAR_T *value)
{
    ty_cJSON *cfg_json = __tuya_json_parse();
    if (NULL == cfg_json){
        PR_ERR("load configure failed\n");
        return;
    }
    if (TRUE != ty_cJSON_ReplaceItemInObject(cfg_json, key, ty_cJSON_CreateString(value))){
        PR_ERR("no key[%s] in db cfg",key);
        ty_cJSON_AddItemToObject(cfg_json, key, ty_cJSON_CreateString(value));
    }else{
        PR_DEBUG("replace key succ[%s]",key);
    }
    CHAR_T *json_tmp = ty_cJSON_Print(cfg_json);
    PR_DEBUG("INFO2[%s]",json_tmp);
    __tuya_json_write(json_tmp);
    ty_cJSON_free(json_tmp);
    ty_cJSON_Delete(cfg_json);
}

STATIC INT_T __tuya_app_read_STR(CHAR_T *key, CHAR_T *value, INT_T value_size)
{
    memset(value, 0, value_size);
    ty_cJSON *cfg_json = __tuya_json_parse();
    if (NULL == cfg_json){
        PR_ERR("load configure failed\n");
        return OPRT_CJSON_PARSE_ERR;
    }
    INT_T ret = 0;
    ty_cJSON* val = ty_cJSON_GetObjectItem(cfg_json, key);
    if (val) {
        strncpy(value, val->valuestring, value_size);
        ret = 0;
    }else{
        ret = -1;
    }
    ty_cJSON_Delete(cfg_json);
    return ret;

}

#ifdef TUYA_DP_SLEEP_MODE
VOID IPC_APP_set_sleep_mode(BOOL_T sleep_mode)
{
    PR_INFO("set sleep_mode:%d \r\n", sleep_mode);
    //TODO
    /* sleep mode,BOOL type,true means sleep,false means wake */

    __tuya_app_write_INT("tuya_sleep_mode", sleep_mode);
}

BOOL_T IPC_APP_get_sleep_mode(VOID)
{
    BOOL_T sleep_mode = __tuya_app_read_INT("tuya_sleep_mode");
    PR_INFO("curr sleep_mode:%d \r\n", sleep_mode);
    return sleep_mode;
}
#endif

//------------------------------------------

#ifdef TUYA_DP_LIGHT
VOID IPC_APP_set_light_onoff(BOOL_T light_on_off)
{
    PR_INFO("set light_on_off:%d \r\n", light_on_off);
    //TODO
    /* Status indicator,BOOL type,true means on,false means off */

    __tuya_app_write_INT("tuya_light_onoff", light_on_off);
}

BOOL_T IPC_APP_get_light_onoff(VOID)
{
    BOOL_T light_on_off = __tuya_app_read_INT("tuya_light_onoff");
    PR_INFO("curr light_on_off:%d \r\n", light_on_off);
    return light_on_off;
}
#endif

//------------------------------------------

#ifdef TUYA_DP_FLIP
VOID IPC_APP_set_flip_onoff(BOOL_T flip_on_off)
{
    PR_INFO("set flip_on_off:%d \r\n", flip_on_off);
    //TODO
    /* flip state,BOOL type,true means inverse,false means normal */

    __tuya_app_write_INT("tuya_flip_onoff", flip_on_off);
}

BOOL_T IPC_APP_get_flip_onoff(VOID)
{
    BOOL_T flip_on_off = __tuya_app_read_INT("tuya_flip_onoff");
    PR_INFO("curr flip_on_off:%d \r\n", flip_on_off);
    return flip_on_off;
}
#endif

//------------------------------------------

#ifdef TUYA_DP_WATERMARK
VOID IPC_APP_set_watermark_onoff(BOOL_T watermark_on_off)
{
    PR_INFO("set watermark_on_off:%d \r\n", watermark_on_off);
    //TODO
    /* Video watermarking (OSD),BOOL type,true means adding watermark on,false means not */

    __tuya_app_write_INT("tuya_watermark_onoff", watermark_on_off);
}

BOOL_T IPC_APP_get_watermark_onoff(VOID)
{
    BOOL_T watermark_on_off = __tuya_app_read_INT("tuya_watermark_onoff");
    PR_INFO("curr watermark_on_off:%d \r\n", watermark_on_off);
    return watermark_on_off;
}
#endif

//------------------------------------------

#ifdef TUYA_DP_WDR
VOID IPC_APP_set_wdr_onoff(BOOL_T wdr_on_off)
{
    PR_INFO("set wdr_on_off:%d \r\n", wdr_on_off);
    //TODO
    /* Wide Dynamic Range Model,BOOL type,true means on,false means off */

    __tuya_app_write_INT("tuya_wdr_onoff", wdr_on_off);

}

BOOL_T IPC_APP_get_wdr_onoff(VOID)
{
    BOOL_T wdr_on_off = __tuya_app_read_INT("tuya_wdr_onoff");
    PR_INFO("curr watermark_on_off:%d \r\n", wdr_on_off);
    return wdr_on_off;
}
#endif

//------------------------------------------

#ifdef TUYA_DP_NIGHT_MODE
STATIC CHAR_T s_night_mode[4] = {0};//for demo
VOID IPC_APP_set_night_mode(CHAR_T *p_night_mode)
{//0-automatic 1-off 2-on
    PR_INFO("set night_mode:%s \r\n", p_night_mode);
    //TODO
    /* Infrared night vision function,ENUM type*/

    __tuya_app_write_STR("tuya_night_mode", p_night_mode);
}

CHAR_T *IPC_APP_get_night_mode(VOID)
{
    __tuya_app_read_STR("tuya_night_mode", s_night_mode, 4);
    PR_INFO("curr IPC_APP_get_night_mode:%s  \r\n", s_night_mode );
    return  s_night_mode;
}

#endif


//------------------------------------------

#ifdef TUYA_DP_ALARM_FUNCTION
VOID IPC_APP_set_alarm_function_onoff(BOOL_T alarm_on_off)
{
    PR_INFO("set alarm_on_off:%d \r\n", alarm_on_off);
    /* motion detection alarm switch,BOOL type,true means on,false means off.
     * This feature has been implemented, and developers can make local configuration settings and properties.*/

    __tuya_app_write_INT("tuya_alarm_on_off", alarm_on_off);
}

BOOL_T IPC_APP_get_alarm_function_onoff(VOID)
{
    BOOL_T alarm_on_off = __tuya_app_read_INT("tuya_alarm_on_off");
    PR_INFO("curr alarm_on_off:%d \r\n", alarm_on_off);
    return alarm_on_off;
}
#endif

//------------------------------------------

#ifdef TUYA_DP_ALARM_SENSITIVITY
STATIC CHAR_T s_alarm_sensitivity[4] = {0};//for demo
VOID IPC_APP_set_alarm_sensitivity(CHAR_T *p_sensitivity)
{
    PR_INFO("set alarm_sensitivity:%s \r\n", p_sensitivity);
    //TODO
    /* Motion detection alarm sensitivity,ENUM type,0 means low sensitivity,1 means medium sensitivity,2 means high sensitivity*/

    __tuya_app_write_STR("tuya_alarm_sen", p_sensitivity);
}

CHAR_T *IPC_APP_get_alarm_sensitivity(VOID)
{
    __tuya_app_read_STR("tuya_alarm_sen", s_alarm_sensitivity, 4);
    PR_INFO("curr alarm_sensitivity:%s \r\n", s_alarm_sensitivity);
    return s_alarm_sensitivity;
}
#endif

#ifdef TUYA_DP_ALARM_ZONE_ENABLE
VOID IPC_APP_set_alarm_zone_onoff(BOOL_T alarm_zone_on_off)
{
    /* Motion detection area setting switch,BOOL type,true means on,false is off*/
    PR_INFO("set alarm_zone_onoff:%d \r\n", alarm_zone_on_off);
  
    /* This feature has been implemented, and developers can make local configuration settings and properties.*/

    __tuya_app_write_INT("alarm_zone_on_off", alarm_zone_on_off);

}

BOOL_T IPC_APP_get_alarm_zone_onoff(VOID)
{
    BOOL_T alarm_zone_on_off = __tuya_app_read_INT("alarm_zone_on_off");
    PR_INFO("curr alarm_on_off:%d \r\n", alarm_zone_on_off);
    return alarm_zone_on_off;
}
#endif


#ifdef TUYA_DP_ALARM_ZONE_DRAW

#define MAX_ALARM_ZONE_NUM      (6)     //Supports the maximum number of detection areas
//Detection area structure
typedef struct{
    char pointX;    //Starting point x  [0-100]
    char pointY;    //Starting point Y  [0-100]
    char width;     //width    [0-100]
    char height;    //height    [0-100]
}ALARM_ZONE_T;

typedef struct{
    int iZoneNum;   //Number of detection areas
    ALARM_ZONE_T alarmZone[MAX_ALARM_ZONE_NUM];
}ALARM_ZONE_INFO_T;


VOID IPC_APP_set_alarm_zone_draw(cJSON * p_alarm_zone)
{
    if (NULL == p_alarm_zone){
        return ;
    }
    #if 0
    /*demo code*/
    /*Motion detection area setting switch*/
    PR_INFO("%s %d set alarm_zone_set:%s \r\n",__FUNCTION__,__LINE__, (char *)p_alarm_zone);
    ALARM_ZONE_INFO_T strAlarmZoneInfo;
    INT_T i = 0;
    cJSON * pJson = cJSON_Parse((CHAR_T *)p_alarm_zone);

    if (NULL == pJson){
        PR_INFO("%s %d step error\n",__FUNCTION__,__LINE__);
        //free(pResult);
        return;
    }
    cJSON * tmp = cJSON_GetObjectItem(pJson, "num");
    if (NULL == tmp){
        PR_INFO("%s %d step error\n",__FUNCTION__,__LINE__);
        cJSON_Delete(pJson);
        //free(pResult);
        return ;
    }
    memset(&strAlarmZoneInfo, 0x00, sizeof(ALARM_ZONE_INFO_T));
    strAlarmZoneInfo.iZoneNum = tmp->valueint;
    PR_INFO("%s %d step num[%d]\n",__FUNCTION__,__LINE__,strAlarmZoneInfo.iZoneNum);
    if (strAlarmZoneInfo.iZoneNum > MAX_ALARM_ZONE_NUM){
        PR_INFO("#####error zone num too big[%d]\n",strAlarmZoneInfo.iZoneNum);
        cJSON_Delete(pJson);
        //free(pResult);
        return ;
    }
    for (i = 0; i < strAlarmZoneInfo.iZoneNum; i++){
        char region[12] = {0};
        cJSON * cJSONRegion = NULL;
        snprintf(region, 12, "region%d",i);
        cJSONRegion = cJSON_GetObjectItem(pJson, region);
        if (NULL == cJSONRegion){
            PR_INFO("#####[%s][%d]error\n",__FUNCTION__,__LINE__);
            cJSON_Delete(pJson);
            //free(pResult);
            return;
        }
        strAlarmZoneInfo.alarmZone[i].pointX = cJSON_GetObjectItem(cJSONRegion, "x")->valueint;
        strAlarmZoneInfo.alarmZone[i].pointY = cJSON_GetObjectItem(cJSONRegion, "y")->valueint;
        strAlarmZoneInfo.alarmZone[i].width = cJSON_GetObjectItem(cJSONRegion,  "xlen")->valueint;
        strAlarmZoneInfo.alarmZone[i].height = cJSON_GetObjectItem(cJSONRegion, "ylen")->valueint;
        PR_INFO("#####[%s][%d][%d,%d,%d,%d]\n",__FUNCTION__,__LINE__,strAlarmZoneInfo.alarmZone[i].pointX,\
            strAlarmZoneInfo.alarmZone[i].pointY,strAlarmZoneInfo.alarmZone[i].width,strAlarmZoneInfo.alarmZone[i].height);
    }
    cJSON_Delete(pJson);
    //free(pResult);
    #endif
    return ;
}

VOID IPC_APP_get_alarm_zone_draw(char * p_alarm_zone)
{
    /*demo code*/
    int i;
    ALARM_ZONE_INFO_T strAlarmZoneInfo;

    memset(&strAlarmZoneInfo, 0x00, sizeof(ALARM_ZONE_INFO_T));
    strAlarmZoneInfo.iZoneNum = 1;
    strAlarmZoneInfo.alarmZone[0].pointX = 0;
    strAlarmZoneInfo.alarmZone[0].pointY = 0;
    strAlarmZoneInfo.alarmZone[0].width = 100;
    strAlarmZoneInfo.alarmZone[0].height = 100;
    /*get param of alarmzoneInfo yourself*/
    if (strAlarmZoneInfo.iZoneNum > MAX_ALARM_ZONE_NUM){
        PR_ERR("[%s] [%d ]get iZoneNum[%d] error",__FUNCTION__,__LINE__,strAlarmZoneInfo.iZoneNum);
        return ;
    }
    for (i = 0; i < strAlarmZoneInfo.iZoneNum; i++){
        char region[64] = {0};
        //{"169":"{\"num\":1,\"region0\":{\"x\":0,\"y\":0,\"xlen\":50,\"ylen\":50}}"}
        if (0 == i){
            snprintf(p_alarm_zone, 256,"{\"num\":%d",strAlarmZoneInfo.iZoneNum);
        }
        snprintf(region, 64, ",\"region%d\":{\"x\":%d,\"y\":%d,\"xlen\":%d,\"ylen\":%d}",i,strAlarmZoneInfo.alarmZone[i].pointX,\
            strAlarmZoneInfo.alarmZone[i].pointY,strAlarmZoneInfo.alarmZone[i].width,strAlarmZoneInfo.alarmZone[i].height);
        strcat(p_alarm_zone, region);
        if(i == (strAlarmZoneInfo.iZoneNum - 1)){
            strcat(p_alarm_zone, "}");
        }
    }
    return ;
}
#endif

#ifdef TUYA_DP_SD_STATUS_ONLY_GET
INT_T IPC_APP_get_sd_status(VOID)
{
    INT_T sd_status = 1;
    /* SD card status, VALUE type, 1-normal, 2-anomaly, 3-insufficient space, 4-formatting, 5-no SD card */
    /* Developer needs to return local SD card status */
    //TODO
    sd_status = 1;

    PR_INFO("curr sd_status:%d \r\n", sd_status);
    return sd_status;
}
#endif

//------------------------------------------

#ifdef TUYA_DP_SD_STORAGE_ONLY_GET
VOID IPC_APP_get_sd_storage(UINT_T *p_total, UINT_T *p_used, UINT_T *p_empty)
{//unit is kb
    //TODO
    /* Developer needs to return local SD card status */
    *p_total = 128 * 1000 * 1000;
    *p_used = 32 * 1000 * 1000;
    *p_empty = *p_total - *p_used;

    PR_INFO("curr sd total:%u used:%u empty:%u \r\n", *p_total, *p_used, *p_empty);
}
#endif
//------------------------------------------

#ifdef TUYA_DP_SD_RECORD_MODE
VOID IPC_APP_set_sd_record_mode(UINT_T sd_record_mode)
{
    PR_INFO("set sd_record_mode:%d \r\n", sd_record_mode);
    if(0 == sd_record_mode)
    {
         tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_EVENT);
    }
    else if(1 == sd_record_mode)
    {
         tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_ALL);
    }
    else
    {
        PR_INFO("Error, should not happen\r\n");
        tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_ALL);
    }
    __tuya_app_write_INT("tuya_sd_record_mode", sd_record_mode);
}

UINT_T IPC_APP_get_sd_record_mode(VOID)
{
    BOOL_T sd_record_mode = __tuya_app_read_INT("tuya_sd_record_mode");
    PR_TRACE("curr sd_record_mode:%d \r\n", sd_record_mode);
    return sd_record_mode;
}
#endif
//------------------------------------------

#ifdef TUYA_DP_SD_RECORD_ENABLE
BOOL_T IPC_APP_get_sd_record_onoff(VOID)
{
    BOOL_T sd_record_on_off  = __tuya_app_read_INT("tuya_sd_record_on_off");
    PR_TRACE("curr sd_record_on_off:%d \r\n", sd_record_on_off);
    return sd_record_on_off;
}
VOID IPC_APP_set_sd_record_onoff(BOOL_T sd_record_on_off)
{
    PR_INFO("set sd_record_on_off:%d \r\n", sd_record_on_off);
    /* SD card recording function swith, BOOL type, true means on, false means off.
     * This function has been implemented, and developers can make local configuration settings and properties.*/

    if(sd_record_on_off == TRUE)
    {
         IPC_APP_set_sd_record_mode( IPC_APP_get_sd_record_mode()  );
    }else
    {
        tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_NONE);
    }

    __tuya_app_write_INT("tuya_sd_record_on_off", sd_record_on_off);
}
#endif

//------------------------------------------

#ifdef TUYA_DP_SD_UMOUNT
BOOL_T IPC_APP_unmount_sd_card(VOID)
{
    BOOL_T umount_ok = TRUE;

    //TODO
    /* unmount sdcard */

    PR_INFO("unmount result:%d \r\n", umount_ok);
    return umount_ok;
}
#endif

//------------------------------------------

#ifdef TUYA_DP_SD_FORMAT
/* -2000: SD card is being formatted, -2001: SD card formatting is abnormal, -2002: No SD card, 
   -2003: SD card error. Positive number is formatting progress */
STATIC INT_T s_sd_format_progress = 0;
void *thread_sd_format(void *arg)
{
    /* First notify to app, progress 0% */
    s_sd_format_progress = 0;
    IPC_APP_report_sd_format_status(s_sd_format_progress);
    tal_system_sleep(1000);

    /* Stop local SD card recording and playback, progress 10%*/
    s_sd_format_progress = 10;
    IPC_APP_report_sd_format_status(s_sd_format_progress);
    tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_NONE);
    tuya_ipc_ss_pb_stop_all();
    tal_system_sleep(1000);

    /* Delete the media files in the SD card, the progress is 30% */
    s_sd_format_progress = 30;
    IPC_APP_report_sd_format_status(s_sd_format_progress);
    //tuya_ipc_ss_delete_all_files();
    tal_system_sleep(1000);

    /* Perform SD card formatting operation */
    //tuya_ipc_sd_format();

    s_sd_format_progress = 80;
    IPC_APP_report_sd_format_status(s_sd_format_progress);
    //TODO
    tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_ALL);
//    IPC_APP_set_sd_record_onoff( IPC_APP_get_sd_record_onoff());

    tal_system_sleep(1000);
    IPC_APP_report_sd_storage();
    /* progress 100% */
    s_sd_format_progress = 100;
    IPC_APP_report_sd_format_status(s_sd_format_progress);

    pthread_exit(0);
}

VOID IPC_APP_format_sd_card(VOID)
{
    PR_INFO("start to format sd_card \r\n");
    /* SD card formatting.
     * The SDK has already completed the writing of some of the code, 
     and the developer only needs to implement the formatting operation. */

    pthread_t sd_format_thread;
    pthread_create(&sd_format_thread, NULL, thread_sd_format, NULL);
    pthread_detach(sd_format_thread);
}
#endif

#ifdef TUYA_DP_SD_FORMAT_STATUS_ONLY_GET
INT_T IPC_APP_get_sd_format_status(VOID)
{
    return s_sd_format_progress;
}
#endif

//------------------------------------------

#ifdef TUYA_DP_PTZ_CONTROL
VOID IPC_APP_ptz_start_move(CHAR_T *p_direction)
{
    PR_INFO("ptz start move:%s \r\n", p_direction);
    //0-up, 1-upper right, 2-right, 3-lower right, 4-down, 5-down left, 6-left, 7-top left

}
#endif

#ifdef TUYA_DP_PTZ_STOP
VOID IPC_APP_ptz_stop_move(VOID)
{
    PR_INFO("ptz stop move \r\n");
    //TODO
    /* PTZ rotation stops */

}
#endif

#ifdef TUYA_DP_PTZ_CHECK
void IPC_APP_ptz_check(VOID)
{
    PR_INFO("ptz check \r\n");
}
#endif

#ifdef TUYA_DP_TRACK_ENABLE
void IPC_APP_track_enable(BOOL_T track_enable)
{
    PR_INFO("track_enable %d\r\n",track_enable);
}

BOOL_T IPC_APP_get_track_enable(void)
{
    char track_enable = 0;
    //the value you get yourself
    return (BOOL_T)track_enable;
}

#endif

#ifdef TUYA_DP_HUM_FILTER
void IPC_APP_human_filter(BOOL_T filter_enable)
{
    PR_INFO("filter_enable %d\r\n",filter_enable);
    return;
}
#endif

#ifdef TUYA_DP_PATROL_MODE
void IPC_APP_set_patrol_mode(BOOL_T patrol_mode)
{
    PR_INFO("patrol_mode %d\r\n",patrol_mode);
    return;
}

char IPC_APP_get_patrol_mode(void)
{
    char patrol_mode = 0;
    //the value you get yourself
    return patrol_mode;
}

#endif

#ifdef TUYA_DP_PATROL_SWITCH
void IPC_APP_set_patrol_switch(BOOL_T patrol_switch)
{
    PR_INFO("patrol_switch %d\r\n",patrol_switch);
    return;
}

BOOL_T IPC_APP_get_patrol_switch(void)
{
    char patrol_switch = 0;
    //the value you get yourself
    return (BOOL_T)patrol_switch;
}

void IPC_APP_ptz_preset_reset(S_PRESET_CFG_T *preset_cfg)
{
    /*Synchronize data from server*/
    return;
}

#endif

#ifdef TUYA_DP_PATROL_TMODE
void IPC_APP_set_patrol_tmode(BOOL_T patrol_tmode)
{
    PR_INFO("patrol_tmode %d\r\n",patrol_tmode);
    return;
}

char IPC_APP_get_patrol_tmode(void)
{
    char patrol_tmode = 0;
    //the value you get yourself
    return patrol_tmode;
}

#endif

#ifdef TUYA_DP_PATROL_TIME
void IPC_APP_set_patrol_time(cJSON * p_patrol_time)
{
    //set your patrol_time
    /*

    cJSON * pJson = cJSON_Parse((CHAR_T *)p_patrol_time);
    if (NULL == pJson){
        TYWARN("----error---\n");

        return -1;
    }
    cJSON* t_start = cJSON_GetObjectItem(pJson, "t_start");
    cJSON* t_end = cJSON_GetObjectItem(pJson, "t_end");
    if ((NULL == t_start) || (NULL == t_end)){
        TYWARN("----t_start---\n");
        cJSON_Delete(pJson);
        return -1;
    }
    PR_DEBUG("stare%s--end:%s\n", t_start->valuestring,t_end->valuestring);

    */
    return;
}

#endif

#ifdef TUYA_DP_PRESET_SET
void IPC_APP_set_preset(cJSON * p_preset_param)
{
    //preset add ,preset del, preset go
#if 0
    cJSON * pJson = cJSON_Parse((CHAR_T *)p_preset_param);
    if (NULL == pJson){
        PR_ERR("null preset set input");
        return -1;
    }
    cJSON* type = cJSON_GetObjectItem(pJson, "type");
    cJSON* data = cJSON_GetObjectItem(pJson, "data");
    if ((NULL == type) || (NULL == data)){
        PR_ERR("invalid preset set input");
        return -1;
    }
   

    PR_DEBUG("preset set type: %d",type->valueint);
    //1:add preset point 2:delete preset point 3:call preset point
    if(type->valueint == 1)
    {
        char pic_buffer[PRESET_PIC_MAX_SIZE] = {0};  
        int pic_size = sizeof(pic_buffer);  /*Image to be shown*/
        S_PRESET_POSITION preset_pos;
        char respond_add[128] = {0}; 
        /*mpId is 1,2,3,4,5,6，The server will generate a set of its own preset point number information based on the mpid.*/
        preset_pos.mpId = 1;   
        preset_pos.ptz.pan = 100; /*horizontal position*/
        preset_pos.ptz.tilt = 60;/*vertical position*/
        cJSON* name = cJSON_GetObjectItem(data, "name");
        int name_len = 0;
        int error_num = 0;
        
        if(name == NULL)
        {
            PR_ERR("name is null\n");
            return -1;
        }
        name_len = strlen(name->valuestring);
        name_len = (name_len > 30)?30:name_len;
        memcpy(&preset_pos.name,name->valuestring,(name_len));
        preset_pos.name[name_len] = '\0';
        error_num = tuya_ipc_preset_add(&preset_pos);

        snprintf(respond_add,128,"{\"type\":%d,\"data\":{\"error\":%d}}",type->valueint,error_num);

        tuya_ipc_dp_report(NULL, TUYA_DP_PRESET_SET,PROP_STR,respond_add,1);

        //tuya_ipc_preset_add_pic(pic_buffer,pic_size); /*if you need show pic ,you should set this api*/
    }
    else if(type->valueint == 2)
    {
        cJSON* num = cJSON_GetObjectItem(data, "num"); //can delete one or more
        cJSON* sets = cJSON_GetObjectItem(data, "sets");
        char respond_del[128] = {0}; 
        cJSON* del_data;
        int del_num = num->valueint;
        for(i = 0; i < del_num; i++)
        {
            del_data = cJSON_GetArrayItem(sets,i);
            cJSON* devId = cJSON_GetObjectItem(del_data, "devId");  /*devid is the preset point number registered in the server*/
            cJSON* mpId = cJSON_GetObjectItem(del_data, "mpId");  /*mpid is the preset point number managed on the device*/
            if((NULL == devId) || (NULL == mpId))
            {
                PR_INFO("devid or mpid is error\n");
                return -1;
            }
            del_preset.seq = atoi(mpId->valuestring);
            
            PR_INFO("%d---%s\n",del_preset.seq,devId->valuestring);

            error_num = (int)time(NULL);

            tuya_ipc_preset_del(devId->valuestring);
            
            snprintf(respond_add,128,"{\"type\":%d,\"data\":{\"error\":%d}}",type->valueint,error_num);
        }
    }
    else if(type->valueint == 3)
    {
        cJSON* mpId = cJSON_GetObjectItem(data, "mpId");

        preset_seq = atoi(mpId->valuestring);
        //get your seq pos and go there
    }
#endif
    return;
}

#endif

#ifdef TUYA_DP_PATROL_STATE
void IPC_APP_patrol_state(int *patrol_state)
{
    //PR_INFO("patrol_state %d\r\n",atoi(patrol_state));
    //return your patrol_state
    return;
}

#endif


#ifdef TUYA_DP_LINK_MOVE_SET
VOID IPC_APP_set_link_pos(INT_T bind_seq)
{
    /*set the link pos*/
    PR_INFO("IPC_APP_set_bind_pos:%d \r\n", bind_seq);
    /*demo
    step1: get the current position
    step2: save the position to flash
    */
    return;
}
#endif


#ifdef TUYA_DP_LINK_MOVE_ACTION
VOID IPC_APP_set_link_move(INT_T bind_seq)
{
    /*move to the link pos*/
    PR_INFO("IPC_APP_set_bind_move:%d \r\n", bind_seq);
    /*demo
     step1: get the position base seq
     step2: go to the target position
    */
    return;
}
#endif

//------------------------------------------

#ifdef TUYA_DP_BLUB_SWITCH
VOID IPC_APP_set_blub_onoff(BOOL_T blub_on_off)
{
    PR_INFO("set blub_on_off:%d \r\n", blub_on_off);
    //TODO
    /* light control switch, BOOL type,true means on,false menas off */

    __tuya_app_write_INT("tuya_blub_on_off", blub_on_off);
}

BOOL_T IPC_APP_get_blub_onoff(VOID)
{
    BOOL_T blub_on_off = __tuya_app_read_INT("tuya_blub_on_off");
    PR_INFO("curr blub_on_off:%d \r\n", blub_on_off);
    return blub_on_off;
}
#endif

//------------------------------------------

#ifdef TUYA_DP_ELECTRICITY
INT_T IPC_APP_get_battery_percent(VOID)
{
    //TODO
    /* battery power percentage VALUE type,[0-100] */

    return 80;
}
#endif

#ifdef TUYA_DP_POWERMODE
CHAR_T *IPC_APP_get_power_mode(VOID)
{
    //TODO
    /* Power supply mode, ENUM type, 
    "0" is the battery power supply state, "1" is the plug-in power supply state (or battery charging state) */

    return "1";
}
#endif

#ifdef TUYA_DP_MOVE_POINT
STATIC UINT64_T  tmp_move_point_pathId = 0;
STATIC UINT64_T  tmp_move_point_id = 0;
STATIC BOOL_T    tmp_move_point_task_flag = FALSE;
VOID * move_point_async(VOID * args)
{
    PR_DEBUG("move point task run [%llu---%llu]", tmp_move_point_pathId, tmp_move_point_id);
    //todo  客户再此处控制设备运动到对应点位
    INT_T  cnt = 0;
    while(TRUE == tmp_move_point_task_flag && cnt < 10){
        sleep(1);
        cnt++;
    }
    CHAR_T  return_message[128] = {0};
    memset(return_message, 0x00, sizeof(return_message));
    snprintf(return_message, 128, "{\"type\":6,\"data\":{\"pathId\":%llu,\"id\":%llu,\"error\":0}}",tmp_move_point_pathId, tmp_move_point_id);
    tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_POINT,PROP_STR,return_message,1);   
    return NULL;    
}
INT_T IPC_APP_set_move_point(CHAR_T * pData)
{
    if (NULL == pData){
        return -1;
    }
    CHAR_T * pJson = ty_cJSON_Parse(pData);
    ty_cJSON *child = ty_cJSON_GetObjectItem(pJson, "type");
    if (NULL == child){
        PR_ERR("no type value");
        return -1;
    } 
    ty_cJSON *child_data = ty_cJSON_GetObjectItem(pJson, "data");
    CHAR_T  return_message[128] = {0};
    PR_DEBUG("get move path type[%d]", child->valueint);
    switch (child->valueint)
    {
    case 6:
    {
        //运动到对应点位
        ty_cJSON * pathId_json = ty_cJSON_GetObjectItem(child_data, "pathId");
        ty_cJSON * pointId_json = ty_cJSON_GetObjectItem(child_data, "id");
        PR_DEBUG("pathId [%llu] run to point id [%llu]", (UINT64_T)pathId_json->valuedouble, (UINT64_T)pointId_json->valuedouble);
        tmp_move_point_pathId = (UINT64_T)pathId_json->valuedouble;
        tmp_move_point_id = (UINT64_T)pointId_json->valuedouble;
        //异步处理
        pthread_t point_move_thread;
        int ret = pthread_create(&point_move_thread, NULL, move_point_async, NULL);
        if(ret != 0){
           PR_ERR("point_move_thread ,create fail! ret:%d\n",ret);
        }else{
            tmp_move_point_task_flag = TRUE;
            pthread_detach(point_move_thread);
        }
        break;
    }
    case 7:
    {
        //结束点位运行
        ty_cJSON * pathId_json = ty_cJSON_GetObjectItem(child_data, "pathId");
        ty_cJSON * pointId_json = ty_cJSON_GetObjectItem(child_data, "id");
        PR_DEBUG("pathId [%llu] stop run to point id [%llu]", (UINT64_T)pathId_json->valuedouble, (UINT64_T)pointId_json->valuedouble);  
        //todo  客户按照实际情况去结束正在执行的点位运行，如果没有点位正在执行，上报错误
        tmp_move_point_task_flag = FALSE;
        CHAR_T  return_message[128] = {0};
        memset(return_message, 0x00, sizeof(return_message));
        snprintf(return_message, 128, "{\"type\":7,\"data\":{\"pathId\":%llu,\"id\":%llu,\"error\":0}}",(UINT64_T)pathId_json->valuedouble, pointId_json->valuedouble);
        tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_POINT,PROP_STR,return_message,1);   
        break;
    }
    default:
        PR_ERR("not support [%d]", child->valueint);
        break;
    }

    return 0;
}
#endif

#ifdef TUYA_DP_MOVE_PATH

STATIC UINT64_T  tmp_move_path_task_pathId = 0;
STATIC BOOL_T    tmp_move_path_task_flag = FALSE;
VOID * move_path_async(VOID * args)
{
    PR_DEBUG("move path task run [%llu]", tmp_move_path_task_pathId);
    //todo  客户再此处控制设备按照轨迹执行
    INT_T  cnt = 0;
    while(TRUE == tmp_move_path_task_flag && cnt < 10){
        sleep(1);
        cnt++;
    }
    CHAR_T  return_message[128] = {0};
    memset(return_message, 0x00, sizeof(return_message));
    snprintf(return_message, 128, "{\"type\":3,\"data\":{\"pathId\":%llu,\"error\":0}}",tmp_move_path_task_pathId);
    tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);  
    return NULL;     
}
INT_T IPC_APP_set_move_path(CHAR_T * pData)
{
    if (NULL == pData){
        return -1;
    }
    PR_DEBUG("##### recv cmd [%s]",pData);
    CHAR_T * pJson = ty_cJSON_Parse(pData);

    ty_cJSON *child = ty_cJSON_GetObjectItem(pJson, "type");
    if (NULL == child){
        PR_ERR("no type value");
        return -1;
    } 
    ty_cJSON *child_data = ty_cJSON_GetObjectItem(pJson, "data");
    CHAR_T  return_message[128] = {0};
    PR_DEBUG("get move path type[%d]", child->valueint);
    switch (child->valueint)
    {
    case 1:
    {
        //添加轨迹
        ty_cJSON *pathName = ty_cJSON_GetObjectItem(child_data, "pathName");
        PR_DEBUG("add path name [%s]", pathName->valuestring);
        UINT64_T pathId = 0;
        INT_T err_num = 0;
        if (OPRT_OK != tuya_ipc_move_path_add(pathName->valuestring, &pathId)){
            PR_ERR("move path add failed");
            err_num = 101;
        }else{
            //stor pathId to db
            CHAR_T tmp_value[4096] = {0};
            if (0 != __tuya_app_read_STR("tuya_move_path", tmp_value, sizeof(tmp_value))){
                PR_ERR("no move path info ");
                ty_cJSON * root_tmp = ty_cJSON_CreateObject();
                ty_cJSON * move_path_list_json = ty_cJSON_AddArrayToObject(root_tmp, "pathList");
                ty_cJSON * move_path_json = ty_cJSON_CreateObject();
                ty_cJSON_AddNumberToObject(move_path_json, "pathId", pathId);
                ty_cJSON_AddStringToObject(move_path_json, "pathName", pathName->valuestring);
                ty_cJSON_AddItemToArray(move_path_list_json, move_path_json);
                CHAR_T * json_buff = ty_cJSON_PrintUnformatted(root_tmp);
                __tuya_app_write_STR("tuya_move_path", json_buff);
                ty_cJSON_Delete(root_tmp);
                ty_cJSON_FreeBuffer(json_buff);
            }else{
                PR_DEBUG("write data");
                //有数据，直接插入新成员
                ty_cJSON * root_tmp = ty_cJSON_Parse(tmp_value);
                ty_cJSON * move_path_list_json = ty_cJSON_GetObjectItemCaseSensitive(root_tmp, "pathList");
                if (ty_cJSON_IsArray(move_path_list_json)){
                    PR_DEBUG("write data step");
                    ty_cJSON * move_path_json = ty_cJSON_CreateObject();
                    ty_cJSON_AddNumberToObject(move_path_json, "pathId", pathId);
                    ty_cJSON_AddStringToObject(move_path_json, "pathName", pathName->valuestring);
                    ty_cJSON_AddItemToArray(move_path_list_json, move_path_json);
                }else{
                    PR_ERR("data error");
                }
                CHAR_T * json_buff = ty_cJSON_PrintUnformatted(root_tmp);
                __tuya_app_write_STR("tuya_move_path", json_buff);
                ty_cJSON_Delete(root_tmp);
                ty_cJSON_FreeBuffer(json_buff);
                PR_DEBUG("write data over");
            }
            err_num = 0;
        }
        memset(return_message, 0x00, sizeof(return_message));
        if (0 == err_num){
            snprintf(return_message, 128, "{\"type\":1,\"data\":{\"pathId\":%llu,\"error\":%d}}",pathId, err_num);
        }else{
            snprintf(return_message, 128, "{\"type\":1,\"data\":{\"error\":%d}}",err_num);
        }
        tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
        break;
    }
    case 2:
    {
        //删除轨迹
        INT_T err_num = 0;
        //删除轨迹信息
        ty_cJSON *pathId_json = ty_cJSON_GetObjectItem(child_data, "pathId");
        PR_DEBUG("delete pathId [%llu]", (UINT64_T)pathId_json->valuedouble);
        if (OPRT_OK != tuya_ipc_move_path_delete((UINT64_T)pathId_json->valuedouble)){
            PR_ERR("move path delete failed");
            err_num = 1001;
        }else{
            err_num = 0;
        }
        //删除本地保存信息
        CHAR_T tmp_value[4096] = {0};
        if (0 != __tuya_app_read_STR("tuya_move_path", tmp_value, sizeof(tmp_value))){
            PR_ERR("no move path info check process");
        }else{
            //有数据，直接插入新成员
            ty_cJSON * root_tmp = ty_cJSON_Parse(tmp_value);
            ty_cJSON * move_path_list_json = ty_cJSON_GetObjectItemCaseSensitive(root_tmp, "pathList");
            if (ty_cJSON_IsArray(move_path_list_json)){
                ty_cJSON * move_path = NULL;
                INT_T index = 0;
                ty_cJSON_ArrayForEach(move_path, move_path_list_json){
                    ty_cJSON * pathId_tmp = ty_cJSON_GetObjectItemCaseSensitive(move_path, "pathId");
                    if (ty_cJSON_IsNumber(pathId_tmp)&& pathId_tmp->valuedouble == pathId_json->valuedouble){
                        ty_cJSON_DeleteItemFromArray(move_path_list_json, index);
                    }
                    index++;
                }

            }else{
                PR_ERR("data error");
            }
            CHAR_T * json_buff = ty_cJSON_PrintUnformatted(root_tmp);
            __tuya_app_write_STR("tuya_move_path", json_buff);
            ty_cJSON_Delete(root_tmp);
            ty_cJSON_FreeBuffer(json_buff);
        }
        memset(return_message, 0x00, sizeof(return_message));
        snprintf(return_message, 128, "{\"type\":2,\"data\":{\"pathId\":%llu,\"error\":%d}}",(UINT64_T)pathId_json->valuedouble, err_num);
        tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
        break;
    }
    case 3:
    {
        //执行轨迹
        /* code */
        ty_cJSON *pathId_json = ty_cJSON_GetObjectItem(child_data, "pathId");
        PR_DEBUG("do pathId [%llu]", (UINT64_T)pathId_json->valuedouble);
        tmp_move_path_task_pathId = (UINT64_T)pathId_json->valuedouble;
        tmp_move_path_task_flag = TRUE;
        //todo  执行运动轨迹，建议异步处理，不要阻塞此接口
        pthread_t point_path_thread;
        int ret = pthread_create(&point_path_thread, NULL, move_path_async, NULL);
        if(ret != 0){
           PR_ERR("point_move_thread ,create fail! ret:%d\n",ret);
        }else{
            pthread_detach(point_path_thread);
        }
        break;
    }
    case 4:
    {
        //结束轨迹运行，和 type 3对应
        ty_cJSON *pathId_json = ty_cJSON_GetObjectItem(child_data, "pathId");
        PR_DEBUG("do pathId [%llu]", (UINT64_T)pathId_json->valuedouble);
        //todo  轨迹执行中断
        tmp_move_path_task_flag = FALSE;
        memset(return_message, 0x00, sizeof(return_message));
        snprintf(return_message, 128, "{\"type\":4,\"data\":{\"pathId\":%llu,\"error\":0}}",(UINT64_T)pathId_json->valuedouble);
        tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
        break;
    }
    case 5:
    {
        //轨迹中添加节点
        INT_T err_num = 0;
        TUYA_IPC_MOVE_POINT_T movePoint = {0};
        ty_cJSON *pathId_json = ty_cJSON_GetObjectItem(child_data, "pathId");
        ty_cJSON *pointName_json = ty_cJSON_GetObjectItem(child_data, "pointName");
        PR_DEBUG("pathId [%llu] add point [%s]", (UINT64_T)pathId_json->valuedouble, pointName_json->valuestring);
        //获取pathId下面的所有节点信息
        CHAR_T tmp_value[4096] = {0};
        if (0 != __tuya_app_read_STR("tuya_move_path", tmp_value, sizeof(tmp_value))){
            PR_ERR("no move path info check process");
            memset(return_message, 0x00, sizeof(return_message));
            snprintf(return_message, 128, "{\"type\":5,\"data\":{\"pathId\":%llu,\"error\":101}}",(UINT64_T)pathId_json->valuedouble);
            tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
        }else{
            //有数据，匹配到合适的pathId
            ty_cJSON * root_tmp = ty_cJSON_Parse(tmp_value);
            ty_cJSON * move_path_list_json = ty_cJSON_GetObjectItemCaseSensitive(root_tmp, "pathList");
            if (ty_cJSON_IsArray(move_path_list_json)){
                ty_cJSON * move_path = NULL;
                ty_cJSON * pathId_tmp = NULL;
                ty_cJSON * point_list = NULL;
                BOOL_T matchFlag = FALSE;
                ty_cJSON_ArrayForEach(move_path, move_path_list_json){
                    pathId_tmp = ty_cJSON_GetObjectItemCaseSensitive(move_path, "pathId");
                    if (ty_cJSON_IsNumber(pathId_tmp)&& pathId_tmp->valuedouble == pathId_json->valuedouble){
                        PR_DEBUG("match pathId [%llu], travel point info", (UINT64_T)pathId_tmp->valuedouble);
                        point_list = ty_cJSON_GetObjectItemCaseSensitive(move_path, "pointList");
                        ty_cJSON * point_json = NULL;
                        ty_cJSON_ArrayForEach(point_json, point_list){
                            ty_cJSON * id_json_tmp = ty_cJSON_GetObjectItem(point_json, "id");
                            ty_cJSON * mpId_json_tmp = ty_cJSON_GetObjectItem(point_json, "mpId");
                            PR_DEBUG("point info id-mpId[%llu---%d]", (UINT64_T)id_json_tmp->valuedouble, (UINT64_T)mpId_json_tmp->valueint);
                        }
                        matchFlag = TRUE;
                        break;
                    }
                }
                if (FALSE == matchFlag){
                    memset(return_message, 0x00, sizeof(return_message));
                    snprintf(return_message, 128, "{\"type\":5,\"data\":{\"pathId\":%llu,\"error\":102}}",(UINT64_T)pathId_json->valuedouble);
                    tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                }else{
                    //回复APP中间状态
                    memset(return_message, 0x00, sizeof(return_message));
                    snprintf(return_message, 128, "{\"type\":5,\"data\":{\"pathId\":%llu,\"error\":0}}",(UINT64_T)pathId_json->valuedouble);
                    tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);

                    //获取当前的点位信息，设备自己参考 TUYA_IPC_MOVE_POINT_T 进行填充
                    strncpy(movePoint.name, pointName_json->valuestring, sizeof(movePoint.name));
                    strncpy(movePoint.coords, "chenlb-test-xyz", sizeof(movePoint.coords));
                    srand(time(NULL));
                    movePoint.mpId = rand();
                    //todo 需要自己生成mpId和coord等信息
                    if (OPRT_OK != tuya_ipc_move_point_add(&movePoint)){
                        PR_ERR("point [%s] add failed", movePoint.name);
                        memset(return_message, 0x00, sizeof(return_message));
                        snprintf(return_message, 128, "{\"type\":6,\"data\":{\"pathId\":%llu,\"error\":101}}",(UINT64_T)pathId_json->valuedouble);
                        tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                        break;
                    }else{

                        memset(return_message, 0x00, sizeof(return_message));
                        snprintf(return_message, 128, "{\"type\":6,\"data\":{\"pathId\":%llu,\"id\":%llu,\"error\":0}}",(UINT64_T)pathId_json->valuedouble, movePoint.id);
                        tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);

                        char snap_addr[100*1024] = {0};
                        int snap_size = 100*1024;
                        IPC_APP_get_snapshot(snap_addr,&snap_size);
                        if (OPRT_OK != tuya_ipc_move_point_add_pic(movePoint.mpId, snap_addr, snap_size)){
                            PR_ERR("add pic faild");
                            memset(return_message, 0x00, sizeof(return_message));
                            snprintf(return_message, 128, "{\"type\":7,\"data\":{\"pathId\":%llu,\"id\":%llu,\"error\":101}}",(UINT64_T)pathId_json->valuedouble, movePoint.id);
                            tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                            break;
                        }else{
                            PR_DEBUG("point [%s] add succ  [%llu]", movePoint.name, movePoint.id);
                            memset(return_message, 0x00, sizeof(return_message));
                            snprintf(return_message, 128, "{\"type\":7,\"data\":{\"pathId\":%llu,\"id\":%llu,\"error\":0}}",(UINT64_T)pathId_json->valuedouble, movePoint.id);
                            tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                            err_num = 0;

                            point_list = ty_cJSON_GetObjectItemCaseSensitive(move_path, "pointList");
                            if (NULL == point_list){
                                point_list = ty_cJSON_CreateArray();  
                                ty_cJSON_AddItemToObject(move_path, "pointList", point_list);  
                            }
                            //节点信息保存到配置文件里面
                            ty_cJSON * point_insert_json = ty_cJSON_CreateObject();
                            ty_cJSON_AddNumberToObject(point_insert_json, "id", movePoint.id);
                            ty_cJSON_AddNumberToObject(point_insert_json, "mpId", movePoint.mpId);
                            ty_cJSON_AddStringToObject(point_insert_json, "coords", movePoint.coords);
                            ty_cJSON_AddItemToArray(point_list, point_insert_json);
                            //更新云端轨迹信息
                            TUYA_IPC_MOVE_PATH_T movePathStr = {0};
                            INT_T point_cnt = ty_cJSON_GetArraySize(point_list);
                            PR_DEBUG("point cnt [%d]",point_cnt);
                            movePathStr.pathId = (UINT64_T)pathId_json->valuedouble;
                            ty_cJSON * pathName_json_tmp = ty_cJSON_GetObjectItem(move_path, "pathName");
                            if (NULL == pathName_json_tmp){
                                PR_DEBUG("get pathName failed");
                            }else{
                                PR_DEBUG("get pathName [%s]",pathName_json_tmp->valuestring);
                                strncpy(movePathStr.pathName, pathName_json_tmp->valuestring, sizeof(movePathStr.pathName));
                            }
                            movePathStr.pointCnt = point_cnt;
                            movePathStr.pointList = malloc(sizeof(TUYA_IPC_MOVE_POINT_KEY_INFO_T)*point_cnt);
                            if (NULL == movePathStr.pointList){
                                PR_ERR("malloc failed ");
                                memset(return_message, 0x00, sizeof(return_message));
                                snprintf(return_message, 128, "{\"type\":8,\"data\":{\"pathId\":%llu,\"error\":101}}",(UINT64_T)pathId_json->valuedouble);
                                tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                                break;
                            }else{
                                INT_T index = 0;
                                for (index = 0 ; index < point_cnt; index++){
                                    PR_DEBUG("step [%d]",index);
                                    sleep(1);
                                    ty_cJSON * point_temp_json = ty_cJSON_GetArrayItem(point_list, index);
                                    if (NULL == point_temp_json){
                                        PR_DEBUG("get item failed");
                                        sleep(1);
                                    }else{
                                        PR_DEBUG("step [%d]",index);
                                        sleep(1);
                                        ty_cJSON * tmp_point_id_json = ty_cJSON_GetObjectItem(point_temp_json, "id");
                                        ty_cJSON * tmp_point_mpId_json = ty_cJSON_GetObjectItem(point_temp_json, "mpId");
                                        if (NULL == tmp_point_id_json || NULL == tmp_point_mpId_json){
                                            PR_DEBUG("get id or mpId failed");
                                        }else{
                                            PR_DEBUG("step [%d] id[%llu]",index, (UINT64_T)tmp_point_id_json->valuedouble);
                                            sleep(1);
                                            movePathStr.pointList[index].id = (UINT64_T)tmp_point_id_json->valuedouble;
                                            PR_DEBUG("step [%d]",index);
                                            sleep(1);
                                            movePathStr.pointList[index].mpId = tmp_point_mpId_json->valueint;
                                        }
                                    }
                                }
                                PR_DEBUG("step ");
                                sleep(1);
                                if (OPRT_OK != tuya_ipc_move_path_update(&movePathStr)){
                                    PR_ERR("path update failed");
                                    memset(return_message, 0x00, sizeof(return_message));
                                    snprintf(return_message, 128, "{\"type\":8,\"data\":{\"pathId\":%llu,\"error\":101}}",(UINT64_T)pathId_json->valuedouble);
                                    tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                                    break;
                                }else{
                                    memset(return_message, 0x00, sizeof(return_message));
                                    snprintf(return_message, 128, "{\"type\":8,\"data\":{\"pathId\":%llu,\"error\":0}}",(UINT64_T)pathId_json->valuedouble);
                                    tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                                }  
                            }

                        }
                    }
                }
            }else{
                PR_ERR("data error");
                memset(return_message, 0x00, sizeof(return_message));
                snprintf(return_message, 128, "{\"type\":5,\"data\":{\"pathId\":%llu,\"error\":101}}",(UINT64_T)pathId_json->valuedouble);
                tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
            }
            CHAR_T * json_buff = ty_cJSON_PrintUnformatted(root_tmp);
            __tuya_app_write_STR("tuya_move_path", json_buff);
            ty_cJSON_Delete(root_tmp);
            ty_cJSON_FreeBuffer(json_buff);
        }        
        break;
    }    
    case 9:
    {
        //删除轨迹中单个节点
        ty_cJSON *pathId_json = ty_cJSON_GetObjectItem(child_data, "pathId");
        ty_cJSON *pointId_json = ty_cJSON_GetObjectItem(child_data, "id");
        PR_DEBUG("pathId [%llu] delete point id [%llu]", (UINT64_T)pathId_json->valuedouble, (UINT64_T)pointId_json->valuedouble);
        //获取pathId下面的所有节点信息
        CHAR_T tmp_value[4096] = {0};
        if (0 != __tuya_app_read_STR("tuya_move_path", tmp_value, sizeof(tmp_value))){
            PR_ERR("no move path info check process");
            memset(return_message, 0x00, sizeof(return_message));
            snprintf(return_message, 128, "{\"type\":9,\"data\":{\"pathId\":%llu,\"error\":101}",(UINT64_T)pathId_json->valuedouble);
            tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
        }else{
            //有数据，匹配到合适的pathId
            ty_cJSON * root_tmp = ty_cJSON_Parse(tmp_value);
            ty_cJSON * move_path_list_json = ty_cJSON_GetObjectItemCaseSensitive(root_tmp, "pathList");
            if (ty_cJSON_IsArray(move_path_list_json)){
                ty_cJSON * move_path = NULL;
                ty_cJSON * pathId_tmp = NULL;
                BOOL_T matchFlag = FALSE;
                ty_cJSON_ArrayForEach(move_path, move_path_list_json){
                    pathId_tmp = ty_cJSON_GetObjectItemCaseSensitive(move_path, "pathId");
                    if (ty_cJSON_IsNumber(pathId_tmp)&& pathId_tmp->valuedouble == pathId_json->valuedouble){
                        PR_DEBUG("match pathId [%llu], travel point info", (UINT64_T)pathId_tmp->valuedouble);
                        ty_cJSON * point_list = ty_cJSON_GetObjectItemCaseSensitive(move_path, "pointList");
                        ty_cJSON * point_json = NULL;
                        ty_cJSON_ArrayForEach(point_json, point_list){
                            ty_cJSON * id_json_tmp = ty_cJSON_GetObjectItem(point_json, "id");
                            ty_cJSON * mpId_json_tmp = ty_cJSON_GetObjectItem(point_json, "mpId");
                            PR_DEBUG("point info id-mpId[%llu---%d]", (UINT64_T)id_json_tmp->valuedouble, mpId_json_tmp->valueint);
                        }
                        matchFlag = TRUE;
                        break;
                    }
                }
                if (FALSE == matchFlag){
                    PR_ERR("pathId match failed");
                    memset(return_message, 0x00, sizeof(return_message));
                    snprintf(return_message, 128, "{\"type\":9,\"data\":{\"pathId\":%llu,\"error\":102}}",(UINT64_T)pathId_json->valuedouble);
                    tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                }else{
                    //匹配点位信息，如果觉得本地配置不可靠，可以去云端同步
                    //遍历匹配 point id信息
                    //遍历节点信息
                    BOOL_T matchPointFlag = FALSE;
                    ty_cJSON * point_list_json = ty_cJSON_GetObjectItem(move_path, "pointList");
                    ty_cJSON * move_point = NULL;
                    INT_T point_index_tmp = 0;
                    if (ty_cJSON_IsArray(point_list_json)){
                        ty_cJSON_ArrayForEach(move_point, point_list_json){
                            ty_cJSON * point_Id_tmp = ty_cJSON_GetObjectItem(move_point, "id");
                            if (ty_cJSON_IsNumber(point_Id_tmp)&& point_Id_tmp->valuedouble == pointId_json->valuedouble){
                                PR_DEBUG("match pointId [%llu], travel point info", (UINT64_T)point_Id_tmp->valuedouble);
                                matchPointFlag = TRUE;
                                break;
                            }
                            point_index_tmp++;
                        }
                        if (FALSE == matchPointFlag){
                            PR_ERR("point Id match failed");
                            memset(return_message, 0x00, sizeof(return_message));
                            snprintf(return_message, 128, "{\"type\":9,\"data\":{\"pathId\":%llu,\"error\":104}}",(UINT64_T)pathId_json->valuedouble);
                            tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                            break;
                        }else{
                            //需要删除节点信息
                            tuya_ipc_move_point_delete((UINT64_T)ty_cJSON_GetObjectItem(move_point, "id")->valuedouble);
                            ty_cJSON_DeleteItemFromArray(point_list_json, point_index_tmp);    
                        }
                    }else{
                        PR_ERR("get point list error");
                        memset(return_message, 0x00, sizeof(return_message));
                        snprintf(return_message, 128, "{\"type\":9,\"data\":{\"pathId\":%llu,\"error\":103}}",pathId_json->valuedouble);
                        tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                        break;
                    }
                    //更新云端轨迹信息
                    TUYA_IPC_MOVE_PATH_T movePathStr = {0};
                    point_list_json = ty_cJSON_GetObjectItem(move_path, "pointList");
                    INT_T point_cnt = ty_cJSON_GetArraySize(point_list_json);
                    PR_DEBUG("point cnt [%d]",point_cnt);
                    movePathStr.pathId = pathId_json->valuedouble;
                    strncpy(movePathStr.pathName, ty_cJSON_GetObjectItem(move_path, "pathName")->valuestring, sizeof(movePathStr.pathName));
                    movePathStr.pointCnt = point_cnt;
                    if (0 != point_cnt){
                        movePathStr.pointList = malloc(sizeof(TUYA_IPC_MOVE_POINT_KEY_INFO_T)*point_cnt);
                    }
                    INT_T index = 0;
                    for (index = 0 ; index < point_cnt; index++){
                        ty_cJSON * point_temp_json = ty_cJSON_GetArrayItem(point_list_json, index);
                        movePathStr.pointList[index].id = ty_cJSON_GetObjectItem(point_temp_json, "id")->valuedouble;
                        movePathStr.pointList[index].mpId = ty_cJSON_GetObjectItem(point_temp_json, "mpId")->valueint;
                    }
                    if (OPRT_OK != tuya_ipc_move_path_update(&movePathStr)){
                        PR_ERR("path update failed");
                        memset(return_message, 0x00, sizeof(return_message));
                        snprintf(return_message, 128, "{\"type\":9,\"data\":{\"pathId\":%llu,\"error\":105}}",(UINT64_T)pathId_json->valuedouble);
                        tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                        break;
                    }else{
                        //更新成功之后，在删除本地节点
                        ty_cJSON_DeleteItemFromArray(point_list_json, point_index_tmp); 
                        memset(return_message, 0x00, sizeof(return_message));
                        snprintf(return_message, 128, "{\"type\":10,\"data\":{\"pathId\":%llu,\"error\":0}}",(UINT64_T)pathId_json->valuedouble);
                        tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                    }  
                }
            }else{
                PR_ERR("data error");
                memset(return_message, 0x00, sizeof(return_message));
                snprintf(return_message, 128, "{\"type\":9,\"data\":{\"pathId\":%llu,\"error\":106}}",(UINT64_T)pathId_json->valuedouble);
                tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                break;
            }
            CHAR_T * json_buff = ty_cJSON_PrintUnformatted(root_tmp);
            __tuya_app_write_STR("tuya_move_path", json_buff);
            ty_cJSON_Delete(root_tmp);
            ty_cJSON_FreeBuffer(json_buff);
        }      
        break;
    }
    case 11:
    {
        //修改轨迹名称
        ty_cJSON *pathId_json = ty_cJSON_GetObjectItem(child_data, "pathId");
        ty_cJSON *pathName_json = ty_cJSON_GetObjectItem(child_data, "pathName");
        PR_DEBUG("pathId [%llu] rename [%s]", (UINT64_T)pathId_json->valuedouble, pathName_json->valuestring);
        //获取pathId下面的所有节点信息
        CHAR_T tmp_value[4096] = {0};
        if (0 != __tuya_app_read_STR("tuya_move_path", tmp_value, sizeof(tmp_value))){
            PR_ERR("no move path info check process");
            memset(return_message, 0x00, sizeof(return_message));
            snprintf(return_message, 128, "{\"type\":11,\"data\":{\"pathId\":%llu,\"pathName\":\"%s\",\"error\":101}",pathId_json->valuedouble,pathName_json->valuestring);
            tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
        }else{
            //有数据，匹配到合适的pathId
            ty_cJSON * root_tmp = ty_cJSON_Parse(tmp_value);
            ty_cJSON * move_path_list_json = ty_cJSON_GetObjectItemCaseSensitive(root_tmp, "pathList");
            if (ty_cJSON_IsArray(move_path_list_json)){
                ty_cJSON * move_path = NULL;
                ty_cJSON * pathId_tmp = NULL;
                BOOL_T matchFlag = FALSE;
                ty_cJSON_ArrayForEach(move_path, move_path_list_json){
                    pathId_tmp = ty_cJSON_GetObjectItemCaseSensitive(move_path, "pathId");
                    if (ty_cJSON_IsNumber(pathId_tmp)&& pathId_tmp->valuedouble == pathId_json->valuedouble){
                        PR_DEBUG("match pathId [%llu], travel point info", pathId_tmp->valuedouble);
                        ty_cJSON * point_list = ty_cJSON_GetObjectItemCaseSensitive(move_path, "pointList");
                        ty_cJSON * point_json = NULL;
                        ty_cJSON_ArrayForEach(point_json, point_list){
                            ty_cJSON * id_json_tmp = ty_cJSON_GetObjectItem(point_json, "id");
                            ty_cJSON * mpId_json_tmp = ty_cJSON_GetObjectItem(point_json, "mpId");
                            PR_DEBUG("point info id-mpId[%llu---%d]", id_json_tmp->valuedouble, mpId_json_tmp->valueint);
                        }
                        matchFlag = TRUE;
                        break;
                    }
                }
                if (FALSE == matchFlag){
                    PR_ERR("pathId match failed");
                    memset(return_message, 0x00, sizeof(return_message));
                    snprintf(return_message, 128, "{\"type\":11,\"data\":{\"pathId\":%llu,\"pathName\":\"%s\",\"error\":102}",(UINT64_T)pathId_json->valuedouble,pathName_json->valuestring);
                    tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                    break;
                }else{
                    //匹配点位信息，如果觉得本地配置不可靠，可以去云端同步
                    //修改名称
                    ty_cJSON * path_name_json_tmp = ty_cJSON_GetObjectItemCaseSensitive(move_path, "pathName");
                    strncpy(path_name_json_tmp->valuestring, pathName_json->valuestring, strlen(path_name_json_tmp->valuestring));
                    //更新云端轨迹信息
                    TUYA_IPC_MOVE_PATH_T movePathStr = {0};
                    ty_cJSON * point_list_json = ty_cJSON_GetObjectItem(move_path, "pointList");
                    INT_T point_cnt = ty_cJSON_GetArraySize(point_list_json);
                    PR_DEBUG("point cnt [%d]",point_cnt);
                    movePathStr.pathId = pathId_json->valuedouble;
                    strncpy(movePathStr.pathName, pathName_json->valuestring, sizeof(movePathStr.pathName));
                    PR_DEBUG("update pathName [%s] ", movePathStr.pathName);
                    movePathStr.pointCnt = point_cnt;
                    if (0 != point_cnt){
                        movePathStr.pointList = malloc(sizeof(TUYA_IPC_MOVE_POINT_KEY_INFO_T)*point_cnt);
                    }

                    INT_T index = 0;
                    for (index = 0 ; index < point_cnt; index++){
                        ty_cJSON * point_temp_json = ty_cJSON_GetArrayItem(move_path, index);
                        movePathStr.pointList[index].id = ty_cJSON_GetObjectItem(point_temp_json, "id")->valuedouble;
                        movePathStr.pointList[index].mpId = ty_cJSON_GetObjectItem(point_temp_json, "mpId")->valueint;
                    }
                    if (OPRT_OK != tuya_ipc_move_path_update(&movePathStr)){
                        PR_ERR("path update failed");
                        memset(return_message, 0x00, sizeof(return_message));
                        snprintf(return_message, 128, "{\"type\":11,\"data\":{\"pathId\":%llu,\"pathName\":\"%s\",\"error\":103}}",(UINT64_T)pathId_json->valuedouble,(CHAR_T *)pathName_json->valuestring);
                        tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                        break;
                    }else{
                        memset(return_message, 0x00, sizeof(return_message));
                        snprintf(return_message, 128, "{\"type\":11,\"data\":{\"pathId\":%llu,\"pathName\":\"%s\",\"error\":0}}",(UINT64_T)pathId_json->valuedouble,(CHAR_T *)pathName_json->valuestring);
                        tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                    }  
                }
            }else{
                PR_ERR("data error");
                memset(return_message, 0x00, sizeof(return_message));
                snprintf(return_message, 128, "{\"type\":11,\"data\":{\"pathId\":%llu,\"pathName\":\"%s\",\"error\":104}}",(UINT64_T)pathId_json->valuedouble,(CHAR_T *)pathName_json->valuestring);
                tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
                break;
            }
            CHAR_T * json_buff = ty_cJSON_PrintUnformatted(root_tmp);
            __tuya_app_write_STR("tuya_move_path", json_buff);
            ty_cJSON_Delete(root_tmp);
            ty_cJSON_FreeBuffer(json_buff);
        }            
        break;   
    }
    case 12:
    {
        //保存路径，主要是通知设备当前路径完成操作，可以在此做一些自定义行为
        ty_cJSON *pathId_json = ty_cJSON_GetObjectItem(child_data, "pathId");
        PR_DEBUG("pathId [%llu] save", (UINT64_T)pathId_json->valuedouble);
        memset(return_message, 0x00, sizeof(return_message));
        snprintf(return_message, 128, "{\"type\":12,\"data\":{\"pathId\":%llu,\"error\":0}}",(UINT64_T)pathId_json->valuedouble);
        tuya_ipc_dp_report(NULL, TUYA_DP_MOVE_PATH,PROP_STR,return_message,1);
        break;
    }
    default:
        PR_ERR("not support [%d]", child->valueint);
        break;
    }   
    
}

#endif

#if defined(WIFI_GW) && (WIFI_GW==1)
#ifdef TUYA_DP_AP_MODE
STATIC CHAR_T response[128] = {0};
#define WIFI_SSID_LEN 32    // tuya sdk definition WIFI SSID MAX LEN
#define WIFI_PASSWD_LEN 64  // tuya sdk definition WIFI PASSWD MAX LEN
CHAR_T * IPC_APP_get_ap_mode(VOID)
{
    CHAR_T ap_ssid[WIFI_SSID_LEN + 1] = {0};
    CHAR_T ap_pw[WIFI_PASSWD_LEN + 1] = {0};
    WF_WK_MD_E cur_mode = WWM_STATION;
    NW_MAC_S mac = {{0}};
    tkl_wifi_get_work_mode(&cur_mode);
    INT_T is_ap = 0;
    if(cur_mode == WWM_SOFTAP)
    {
        is_ap = 1;
    }
    OPERATE_RET op_ret = tkl_wifi_get_mac(WF_AP,&mac);
    if(OPRT_OK != op_ret) {
        strncpy(ap_ssid, "TUYA_IPC_AP", WIFI_SSID_LEN);
    }
    else
    {
        snprintf(ap_ssid, WIFI_SSID_LEN, "TUYA_IPC-%02X%02X",mac.mac[4],mac.mac[5]);
    }
    __tuya_app_read_STR("tuya_ap_passwd", ap_pw, WIFI_PASSWD_LEN);

    snprintf(response, 128, "{\"is_ap\":%d,\"ap_ssid\":\"%s\",\"password\":\"%s\"}",is_ap,ap_ssid,ap_pw);
    return response;
}
#endif
#ifdef TUYA_DP_AP_SWITCH
VOID* __ap_change_thread(void *arg)
{
    WF_WK_MD_E cur_mode = WWM_STATION;
    tkl_wifi_get_work_mode(&cur_mode);
    WF_AP_CFG_IF_S ap_cfg;
    INT_T ap_onoff = 0;
    tal_system_sleep(5000);

    ap_onoff = __tuya_app_read_INT("tuya_ap_on_off");
    memset(&ap_cfg, 0, SIZEOF(WF_AP_CFG_IF_S));
    PR_INFO("%s : ap_onoff:%d\n", __func__, ap_onoff);
    if(ap_onoff)
    {
        //ap, sta, monitor, do same
        __tuya_app_read_STR("tuya_ap_ssid", (CHAR_T*)ap_cfg.ssid, WIFI_SSID_LEN);
        __tuya_app_read_STR("tuya_ap_passwd", (CHAR_T*)ap_cfg.passwd, WIFI_PASSWD_LEN);

        ap_cfg.s_len = strlen((CHAR_T *)(ap_cfg.ssid));
        ap_cfg.ssid_hidden = 0;
        ap_cfg.p_len = strlen((CHAR_T *)(ap_cfg.passwd));
        if(ap_cfg.p_len == 0)
        {
            ap_cfg.md = WAAM_OPEN;
        }
        else
        {
            ap_cfg.md = WAAM_WPA2_PSK;
        }

        tkl_wifi_start_ap(&ap_cfg);
    }
    else
    {
        if(cur_mode == WWM_SOFTAP)
        {
            tkl_wifi_stop_ap();
        }
        else
        {
            //not ap mode,do nothing
        }
    }

    return NULL;
}

VOID change_ap_process()
{
    pthread_t ap_change_thread;
    int ret = pthread_create(&ap_change_thread, NULL, __ap_change_thread, NULL);
    if(ret != 0)
    {
       PR_ERR("ap_change_thread ,create fail! ret:%d\n",ret);
       return;
    }

    pthread_detach(ap_change_thread);
}

INT_T IPC_APP_set_ap_mode(IN cJSON *p_ap_info)
{
    if (NULL == p_ap_info){
        return 0;
    }

    INT_T ap_onoff = -1;

    PR_INFO("%s %d handle_DP_AP_SWITCH:%s \r\n",__FUNCTION__,__LINE__, (char *)p_ap_info);

    cJSON * pJson = cJSON_Parse((CHAR_T *)p_ap_info);

    if (NULL == pJson){
        PR_ERR("%s %d step error\n",__FUNCTION__,__LINE__);
        return ap_onoff;
    }
    cJSON * tmp = cJSON_GetObjectItem(pJson, "ap_enable");
    if (NULL == tmp){
        PR_ERR("%s %d step error\n",__FUNCTION__,__LINE__);
        cJSON_Delete(pJson);
        return ap_onoff;
    }
    ap_onoff = tmp->valueint;
    __tuya_app_write_INT("tuya_ap_on_off", ap_onoff);

    if(ap_onoff == 1)
    {
        cJSON *tmpSsid = cJSON_GetObjectItem(pJson, "ap_ssid");
        if (NULL == tmpSsid){
            PR_ERR("%s %d step error\n",__FUNCTION__,__LINE__);
            cJSON_Delete(pJson);
            return ap_onoff;
        }
        PR_INFO("###[%d] get ssid:%s\n", ap_onoff, tmpSsid->valuestring);

        cJSON *tmpPwd = cJSON_GetObjectItem(pJson, "ap_pwd");
        if (NULL == tmpPwd){
            PR_ERR("%s %d step error\n",__FUNCTION__,__LINE__);
            cJSON_Delete(pJson);
            return ap_onoff;
        }
        PR_INFO("###get pwd:%s\n", tmpPwd->valuestring);

        __tuya_app_write_STR("tuya_ap_ssid", tmpSsid->valuestring);
        __tuya_app_write_STR("tuya_ap_passwd", tmpPwd->valuestring);
    }

    cJSON_Delete(pJson);

    return ap_onoff;
}

CHAR_T * IPC_APP_update_ap_mode(INT_T is_ap)
{
    CHAR_T ap_ssid[WIFI_SSID_LEN + 1] = {0};
    CHAR_T ap_pw[WIFI_PASSWD_LEN + 1] = {0};
    NW_MAC_S mac = {{0}};
    OPERATE_RET op_ret = tkl_wifi_get_mac(WF_AP,&mac);
    if(OPRT_OK != op_ret) {
        strncpy(ap_ssid, "TUYA_IPC_AP", WIFI_SSID_LEN);
    }
    else
    {
        snprintf(ap_ssid, WIFI_SSID_LEN, "TUYA_IPC-%02X%02X",mac.mac[4],mac.mac[5]);
    }
    __tuya_app_read_STR("tuya_ap_passwd", ap_pw, WIFI_PASSWD_LEN);

    snprintf(response, 128, "{\"is_ap\":%d,\"ap_ssid\":\"%s\",\"password\":\"%s\"}",is_ap,ap_ssid,ap_pw);
    return response;
}

INT_T IPC_APP_get_ap_mode_config()
{
    return __tuya_app_read_INT("tuya_ap_on_off");
}

INT_T IPC_APP_switch_ap_mode()
{
    WF_AP_CFG_IF_S ap_cfg = {0};
    //ap, sta, monitor, do same
    __tuya_app_read_STR("tuya_ap_ssid", (CHAR_T*)ap_cfg.ssid, WIFI_SSID_LEN);
    __tuya_app_read_STR("tuya_ap_passwd", (CHAR_T*)ap_cfg.passwd, WIFI_PASSWD_LEN);

    ap_cfg.s_len = strlen((CHAR_T *)(ap_cfg.ssid));
    ap_cfg.ssid_hidden = 0;
    ap_cfg.p_len = strlen((CHAR_T *)(ap_cfg.passwd));
    if(ap_cfg.p_len == 0)
    {
        ap_cfg.md = WAAM_OPEN;
    }
    else
    {
        ap_cfg.md = WAAM_WPA2_PSK;
    }

    return tkl_wifi_start_ap(&ap_cfg);

    return 0;
}
#endif

#ifdef TUYA_DP_AP_TIME_SYNC
STATIC BOOL_T time_sync = FALSE;
VOID IPC_APP_set_time(IN CHAR_T *p_time_info)
{
    if (NULL == p_time_info){
        return;
    }
    if(time_sync == TRUE)
    {
        return;
    }        
    PR_INFO("%s %d recv time sync from app:%s \r\n",__FUNCTION__,__LINE__, p_time_info);

    tuya_ipc_set_service_time(atoi(p_time_info));
    time_sync = TRUE;
    return;
}
#endif

#ifdef TUYA_DP_AP_TIME_ZONE
STATIC BOOL_T timezone_sync = FALSE;
VOID IPC_APP_set_time_zone(IN CHAR_T *p_time_info)
{
    if (NULL == p_time_info){
        return;
    }
    PR_INFO("%s %d recv timeZONE sync from app:%s \r\n",__FUNCTION__,__LINE__, p_time_info);
    if(timezone_sync == TRUE)
    {
        return;
    }        

    extern OPERATE_RET tal_time_set_time_zone(IN CONST CHAR_T *time_zone);
    tal_time_set_time_zone(p_time_info);
    
    //TODO: You need call your own timezone setter function here.

    timezone_sync = TRUE;
    return;
}
#endif
#endif

